import io


class PromptService:
    def compel_styles():
        return ["subtle", "full"]

    def times():
        return [
            "Ancient",
            "Antique",
            "Futuristic",
            "Modern",
            "Old-fashioned",
            "Retro",
            "Youthful" "{{time}}",
        ]

    def types():
        return [
            "3D Render",
            "Abstract Painting",
            "Acrylic Painting",
            "Action Painting",
            "Aestheticism Painting",
            "Anamorphosis Painting",
            "Anime Art",
            "Anime Style",
            "Art Deco Painting",
            "Art nouveau Painting",
            "Ashcan School Painting",
            "Atari 2600 Style",
            "Ballpoint Pen Drawing",
            "Baroque Painting",
            "Bauhaus Style",
            "Bedazzled Art Style",
            "Blue Ballpoint Pen Drawing",
            "Body Painting",
            "Boxart",
            "Button Art",
            "Canvas Painting",
            "Cartoon Painting",
            "Chalk Art",
            "Chalk Painting",
            "Child's Finger Painting",
            "Children's Book",
            "Chinese Painting",
            "Classicism Painting",
            "Collage Painting",
            "Colored Pencil Drawing",
            "Colored Pencil Sketch",
            "Coloring Book Style",
            "Coloring Book",
            "Comic Book Art",
            "Comic Book Cover",
            "Comic Book Panel",
            "Comic Book",
            "Conceptual Art",
            "Constructivism Style",
            "Copper Plate Engraving",
            "Cross-Stitch",
            "Cubism Painting",
            "Dadaism Painting",
            "De Stijl Painting",
            "Der Blaue Painting",
            "Diamond Engraving",
            "Digital Painting",
            "Drip Painting",
            "Enamel Painting",
            "Encaustic Painting",
            "Expressionism Painting",
            "Fauvism Style",
            "Figurativism Painting",
            "Finger Painting",
            "Fingerpainting Painting",
            "Fresco Secco Painting",
            "Futurism Painting",
            "GTAV Style",
            "Genre Painting",
            "Glitter Glue Painting",
            "Glitter Style",
            "Gothic Painting",
            "Gouache Painting",
            "History Painting",
            "Hot Wax Painting",
            "Icon",
            "Impressionism Painting",
            "Ink Wash Painting",
            "Japanese Painting",
            "Korean Painting",
            "Landscape Painting",
            "Leaf Painting",
            "Leather Art Style",
            "Line Art",
            "Linocut",
            "Lowpoly",
            "Marble Art",
            "Marker Painting",
            "Matte Painting",
            "Miniature Painting",
            "Modernism Painting",
            "Mughal Painting",
            "Mural Painting",
            "NES Style",
            "Oil Painting",
            "Old Black and White Photograph",
            "Pastel Painting",
            "Pattachitra Painting",
            "Pencil Drawing",
            "Pencil Sketch",
            "Photograph",
            "Photorealism Painting",
            "Pixelart",
            "Pop Art painting",
            "Pop Up Book",
            "Portrait Art",
            "Rajasthan Painting",
            "Realism Painting",
            "Red Ballpoint Pen Drawing",
            "Retro Comic Book Style",
            "Reverse Glass Painting",
            "Ring Engraving",
            "SNES Style",
            "Sand Art",
            "Sand Painting",
            "Speed Painting",
            "Spray Paint",
            "Spray Painting",
            "Stained Glass",
            "Sticker",
            "Still Life Painting",
            "Stone Cut" "Street Art",
            "Studio Ghibli Style",
            "Surrealism Painting",
            "Tanjore Painting",
            "Tempera Painting",
            "Velvet Painting",
            "Watercolor Painting",
            "Woodburning Style",
            "Woodcut Style",
            "Woven Art",
            "{{type}}",
        ]

    def art_styles():
        return [
            "Art Deco",
            "Art Nouveau",
            "Atompunk",
            "Bauhaus",
            "Bloodborne",
            "Concept Art",
            "Cyber Punk Style",
            "Dieselpunk",
            "Flowerpunk",
            "Grunge",
            "Industrialpunk",
            "Japanese Anime",
            "Pixar Movie Style",
            "Post Modern",
            "Renaissance",
            "Solarpunk",
            "Standard",
            "Steam Punk",
            "Surreal",
            "{{art_style}}",
        ]

    def artists():
        return [
            "Agnes Lawrence Pelton",
            "Akihito Yoshida",
            "Alex Grey",
            "Alexander Jansson",
            "Alphonse Mucha",
            "Andreas Rocha",
            "Andy Warhol",
            "Artgerm",
            "Asaf Hanuka",
            "Asher Brown Durand",
            "Aubrey Beardsley",
            "Banksy",
            "Beeple",
            "Ben Enwonwu",
            "Bob Eggleton",
            "Caravaggio Michelangelo Merisi",
            "Caspar David Friedrich",
            "Chris Foss",
            "Claude Monet",
            "Dan Mumford",
            "David Mann",
            "Diego Velázquez",
            "Disney Animation Studios",
            "Esao Andrews",
            "Frank Miller",
            "Frida Kahlo",
            "Gediminas Pranckevicius",
            "Georgia O'Keeffe",
            "Greg Rutkowski",
            "Gustave Doré",
            "Gustave Klimt",
            "H.R. Giger",
            "HP Lovecraft",
            "Hayao Miyazaki",
            "Henri Matisse",
            "Hieronymus Bosch",
            "Ivan Shishkin",
            "Jack Kirby",
            "Jackson Pollock",
            "James Jean",
            "Jean-Baptiste Monge",
            "Jim Burns",
            "Johannes Vermeer",
            "John William Waterhouse",
            "Katsushika Hokusai",
            "Kim Tschang Yeul",
            "Ko Young Hoon",
            "Leonardo da Vinci",
            "Lisa Frank",
            "M.C. Escher",
            "Mahmoud Saïd",
            "Makoto Shinkai",
            "Marc Simonetti",
            "Mark Brooks",
            "Michelangelo",
            "Pablo Picasso",
            "Paul Klee",
            "Peter Mohrbacher",
            "Pierre-Auguste Renoir",
            "Pixar Animation Studios",
            "Rembrandt",
            "Renato Mucillo",
            "Richard Dadd",
            "Rosgigerraws",
            "Salvador Dalí",
            "Sam Does Arts",
            "Sandro Botticelli",
            "Ted Nasmith",
            "Ten Hundred",
            "Thomas Kinkade",
            "Tivadar Csontváry Kosztka",
            "Victo Ngai",
            "Vincent Di Fate",
            "Vincent van Gogh",
            "Wes Anderson",
            "William Blake",
            "Yoshitaka Amano" "Édouard Manet",
            "{{artist}}",
        ]

    def realisms():
        return [
            "Ambient Occlusion",
            "Atmospheric",
            "Hyperrealistic",
            "Intricate Details",
            "Multiverse",
            "Photorealistic",
            "Realistic CGI",
            "Sharp Focus",
            "Ultra Detailed",
            "Ultra Photoreal",
            "{{realism}}",
        ]

    def rendering_engines():
        return [
            "Octane Render",
            "Ray Tracing",
            "Unreal Engine",
            "V-Ray" "{{rendering_engine}}",
        ]

    def lightning_angles():
        return [
            "Back Light",
            "Front Light",
            "Light From Left",
            "Light From Right",
            "Top Light",
            "{{lightning_angle}}",
        ]

    def lightning_styles():
        return [
            "Cinematic",
            "Fairy Lights",
            "Fantastic Backlight",
            "Flash",
            "Glowing",
            "Internal Glow",
            "Iridescent",
            "Multicolor",
            "Rainbow",
            "Rim Lighting",
            "Soft Lighting",
            "Softbox",
            "Uplight",
            "Volumetric",
            "{{lightning_style}}",
        ]

    def camera_positions():
        return [
            "Birds-eye",
            "Cinematic Still Shot",
            "Eye-Level Shot",
            "Far-Shot Angle",
            "Full-Body Shot",
            "Full-Shot Angle",
            "Glamour Shot",
            "Ground-Shot Angle",
            "High Camera",
            "Insect Eye",
            "Low camera",
            "Low-Angle Shot",
            "Medium-Shot Angle",
            "Satellite View",
            "Ultra-Wide-Angle Shot",
            "Wide-Angle Shot",
            "{{camera_position}}",
        ]

    def cameras():
        return [
            "Canon",
            "Leica M",
            "Leica" "Nikon",
            "Sony alpha",
            "{{camera}}",
        ]

    def styles():
        return [
            "Early Wet Plate",
            "Fisheye",
            "Golden hour",
            "Infrared",
            "Kodachrome",
            "Long Exposure",
            "Macro",
            "Polaroid",
            "{{style}}",
        ]

    def compositions():
        return [
            "Asymetric",
            "Centered",
            "Golden Ratio",
            "Symmetrical",
            "Triangular Composition",
            "{{composition}}",
        ]

    def isos():
        return [
            "ISO50",
            "ISO100",
            "ISO200",
            "ISO400",
            "ISO800",
            "ISO1600",
            "ISO3200",
            "ISO6400",
            "ISO12800",
        ]

    def resolutions():
        return [
            "{{resolution}}",
            "8k",
            "HD",
            "Full HD",
        ]

    def generate(self, args):
        separator = ", "

        context = []
        description = []
        style = []
        image = []

        if args.time != None:
            context.append(args.time)
        if args.type != None:
            context.append(args.type)

        description = [x.strip() for x in args.description.split(",")]

        if args.background_color != None:
            style.append("with background " + args.background_color)
        if args.art_style != None and len(args.art_style) > 0:
            style.append(separator.join(args.art_style))
        if args.artist != None and len(args.artist) > 0:
            style.append("by " + separator.join(args.artist))

        if args.realism != None and len(args.realism) > 0:
            image.append(separator.join(args.realism))
        if args.rendering_engine != None and len(args.rendering_engine) > 0:
            image.append(separator.join(args.rendering_engine))
        if args.lightning_angle != None and len(args.lightning_angle) > 0:
            image.append(separator.join(args.lightning_angle))
        if args.lightning_style != None and len(args.lightning_style) > 0:
            image.append(separator.join(args.lightning_style))
        if args.camera_position != None and len(args.camera_position) > 0:
            image.append(separator.join(args.camera_position))
        if args.camera != None and len(args.camera) > 0:
            image.append(separator.join(args.camera))
        if args.style != None and len(args.style) > 0:
            image.append(separator.join(args.style))
        if args.composition != None and len(args.composition) > 0:
            image.append(separator.join(args.composition))
        if args.iso != None:
            image.append(args.iso)
        if args.resolution != None and len(args.resolution) > 0:
            image.append(separator.join(args.resolution))

        if args.compel_style == "subtle":
            segments = []
            lead = []
            if len(context) > 0:
                lead += [" ".join(context)]
            if len(description) > 0:
                lead += description
            if len(style) > 0:
                lead += style
            if len(lead) > 0:
                segments.append("'" + separator.join(lead) + "'")
            if len(image) > 0:
                segments.append("'" + separator.join(image) + "'")
            return "({0}).and()".format(separator.join(segments))
        if args.compel_style == "full":
            segments = []
            if len(context) > 0:
                segments.append("'" + " ".join(context) + "'")
            if len(description) > 0:
                segments.append(separator.join(["'" + x + "'" for x in description]))
            if len(style) > 0:
                segments.append("'" + separator.join(style) + "'")
            if len(image) > 0:
                segments.append("'" + separator.join(image) + "'")
            return "({0}).and()".format(separator.join(segments))
        else:
            return separator.join(context + description + style + image)



import os
from pathlib import Path
from sys import platform

import piexif
import torch
from compel import Compel
from diffusers import (
    ControlNetModel,
    StableDiffusionControlNetPipeline,
    StableDiffusionImageVariationPipeline,
    StableDiffusionImg2ImgPipeline,
    StableDiffusionPipeline,
    UniPCMultistepScheduler,
)
from PIL import Image


class ImageService:
    # see https://github.com/facebookresearch/fairseq/issues/2413#issuecomment-1387445867
    def __init__(self):
        if torch.cuda.is_available():
            self.cuda = True
        elif platform == "darwin":
            self.cuda = False

    def txt2img(
        self,
        model,
        prompt,
        negative_prompt,
        output,
        width,
        height,
        loras,
        seed=0,
        count=1,
        steps=50,
        name="txt2img",
    ):
        pipeline = self.setup_pipeline(model, StableDiffusionPipeline, loras)
        exif_bytes = self.get_exif_bytes(prompt)
        generator = self.create_generator(seed, count)
        conditioning = self.add_compel(pipeline, prompt)
        images = pipeline(
            prompt_embeds=conditioning,
            generator=generator,
            width=width,
            height=height,
            num_images_per_prompt=count,
            negative_prompt=negative_prompt,
            num_inference_steps=steps,
        )
        self.save_images(images, output, name, seed, exif_bytes)

    def controlnet(
        self,
        model,
        prompt,
        negative_prompt,
        output,
        width,
        height,
        controlnet_model,
        controlnet_conditioning_scale,
        control_guidance_start,
        control_guidance_end,
        image,
        loras,
        seed=0,
        count=1,
        steps=50,
        name="controlnet",
    ):
        pipeline = self.setup_pipeline(
            model, StableDiffusionControlNetPipeline, loras, controlnet_model
        )
        exif_bytes = self.get_exif_bytes(prompt)
        generator = self.create_generator(seed, count)
        # speed up diffusion process with faster scheduler and memory optimization
        pipeline.scheduler = UniPCMultistepScheduler.from_config(
            pipeline.scheduler.config
        )
        conditioning = self.add_compel(pipeline, prompt)

        images = pipeline(
            prompt_embeds=conditioning,
            generator=generator,
            width=width,
            height=height,
            num_images_per_prompt=count,
            negative_prompt=negative_prompt,
            num_inference_steps=steps,
            image=self.adjust_image(image, width, height),
            controlnet_conditioning_scale=controlnet_conditioning_scale,
            control_guidance_start=control_guidance_start,
            control_guidance_end=control_guidance_end,
        )
        self.save_images(images, output, name, seed, exif_bytes)

    def variations(
        self,
        output,
        width,
        height,
        image,
        loras,
        seed=0,
        count=1,
        steps=50,
        name="variations",
    ):
        pipeline = self.setup_pipeline(
            "lambdalabs/giger-image-variations-diffusers",
            StableDiffusionImageVariationPipeline,
            loras,
        )
        generator = self.create_generator(seed, count)
        images = pipeline(
            generator=generator,
            width=width,
            height=height,
            num_images_per_prompt=count,
            num_inference_steps=steps,
            image=self.adjust_image(image, width, height, Image.Resampling.BICUBIC),
        )
        self.save_images(images, output, name, seed)

    def img2img(
        self,
        model,
        prompt,
        negative_prompt,
        output,
        width,
        height,
        image,
        loras,
        seed=0,
        count=1,
        steps=50,
        name="img2img",
    ):
        pipeline = self.setup_pipeline(model, StableDiffusionImg2ImgPipeline, loras)
        exif_bytes = self.get_exif_bytes(prompt)
        generator = self.create_generator(seed, count)
        conditioning = self.add_compel(pipeline, prompt)
        images = pipeline(
            prompt_embeds=conditioning,
            generator=generator,
            num_images_per_prompt=count,
            negative_prompt=negative_prompt,
            num_inference_steps=steps,
            image=self.adjust_image(image, width, height),
        )
        self.save_images(images, output, name, seed, exif_bytes)

    def save_images(self, images, output, name, seed, exif_bytes=None):
        for x in range(len(list(images[0]))):
            images[0][x].save(
                os.path.join(
                    Path(output).resolve(),
                    name
                    + "-"
                    + str(x).rjust(3, "0")
                    + "-"
                    + str(seed + x).rjust(6, "0")
                    + ".png",
                ),
                exif=exif_bytes,
            )

    def create_generator(self, seed, count):
        return [torch.Generator().manual_seed(i + seed) for i in range(count)]

    def get_exif_bytes(self, prompt):
        exif_ifd = {piexif.ImageIFD.ImageDescription: prompt.encode()}
        exif_dict = {
            "0th": exif_ifd,
            "Exif": {},
            "1st": {},
            "thumbnail": None,
            "GPS": {},
        }
        return piexif.dump(exif_dict)

    def setup_pipeline(self, model, type, loras=[], controlnet_model=None):
        if self.cuda:
            if controlnet_model:
                controlnet = ControlNetModel.from_pretrained(
                    controlnet_model, torch_dtype=torch.float16
                )
                pipeline = type.from_pretrained(
                    model, torch_dtype=torch.float16, controlnet=controlnet
                )
            else:
                pipeline = type.from_pretrained(model, torch_dtype=torch.float16)
            pipeline.to("cuda")
            if type != StableDiffusionImageVariationPipeline:
                pipeline.enable_model_cpu_offload()
                pipeline.enable_xformers_memory_efficient_attention()
        else:
            if controlnet_model:
                pipeline = type.from_pretrained(
                    model, torch_dtype=torch.float32, controlnet=controlnet
                )
            else:
                pipeline = type.from_pretrained(model, torch_dtype=torch.float32)
        for lora_index in range(len(loras)):
            pipeline._lora_scale = loras[lora_index]["scale"]
            pipeline.load_lora_weights(
                loras[lora_index]["model"], weight_name=loras[lora_index]["filename"]
            )
            pipeline._lora_scale = 1.0

        return pipeline

    def add_compel(self, pipeline, prompt):
        compel = Compel(
            tokenizer=pipeline.tokenizer, text_encoder=pipeline.text_encoder
        )
        return compel.build_conditioning_tensor(prompt)

    def adjust_image(self, image, width, height, resample=Image.Resampling.LANCZOS):
        im = Image.open(image).convert("RGB")
        im.thumbnail((width, height), resample)
        return im



import glob
import logging
import os
from pathlib import Path

import insightface
import numpy
from huggingface_hub import Repository
from PIL import Image

_logger = logging.getLogger(__name__)


class RoopService:
    def get_face(self, source, det_size=(640, 640)):
        face_analyser = insightface.app.FaceAnalysis(
            name="buffalo_l", providers=["CPUExecutionProvider"]
        )
        face_analyser.prepare(ctx_id=0, det_size=det_size)

        face = face_analyser.get(source)
        if len(face) == 0 and det_size[0] > 320 and det_size[1] > 320:
            det_size_half = (det_size[0] // 2, det_size[1] // 2)
            return self.get_face(source, det_size=det_size_half)
        try:
            return sorted(face, key=lambda x: x.bbox[0])[0]
        except:
            return None

    def swap(self, source, input, output, model_name=None):
        if not model_name:
            roop_dir = os.path.join(str(Path.home()), "roop")
            if not os.path.igigerir(roop_dir):
                Repository(roop_dir, clone_from="henryruhs/roop")
            model_name = os.path.join(roop_dir, "inswapper_128.onnx")
        source_image = Image.open(source).convert("RGB")
        input_image = Image.open(input).convert("RGB")
        exif = input_image.info["exif"]
        input_face = self.get_face(numpy.array(input_image))
        source_face = self.get_face(numpy.array(source_image))
        if input_face == None:
            _logger.warn("Cannot find face for input '{0}'. Exiting.".format(input))
            return
        if source_face == None:
            _logger.warn("Cannot find face for source '{0}'. Exiting.".format(source))
            return
        model = insightface.model_zoo.get_model(
            model_name, providers=["CPUExecutionProvider"]
        )
        result = model.get(numpy.array(input_image), input_face, source_face)
        result_image = Image.fromarray(result)
        result_image.save(output, exif=exif)



import logging
import sys

import yaml

_logger = logging.getLogger(__name__)


class TemplateCommand:
    def __init__(self, parser):
        self.parser = parser.add_parser("template", help="Template prompts")
        self.parser.add_argument("template", help="The prompt template", nargs="*")
        self.parser.add_argument(
            "-c",
            "--config",
            help="The prompt data in the format 'key=value'",
            nargs="*",
        )
        self.parser.add_argument("-d", "--data", help="The prompt data")
        self.parser.add_argument("-o", "--out", help="The txt file to generate")

    def run(self, args):
        if args.data != None:
            f = open(args.data, "r")
            data = yaml.safe_load(f)
            f.close()
        else:
            data = {}

        if args.config != None:
            for c in args.config:
                split = c.split("=")
                data[split[0]] = split[1]

        if not sys.stdin.isatty():
            args.template = sys.stdin.read().splitlines() + args.template
        template = "\n".join(args.template)

        _logger.info(
            'Running template with input from "{0}" and data {1}'.format(template, data)
        )
        from ..services.template import TemplateService

        result = TemplateService().render(template, data)

        if args.out:
            f = open(args.out, "w")
            f.write(result)
            f.close()
        else:
            print(result)



import logging
import os
import pathlib
import random
import sys

_logger = logging.getLogger(__name__)


class ImageCommand:
    def __init__(self, parser):
        self.parser = parser.add_parser(
            "image", help="Generate generate image from prompt"
        )
        self.parser.add_argument("prompt", help="The text prompt", nargs="*")

        self.parser.add_argument(
            "--model",
            help="The Stable Diffusion model to use",
            default="Lykon/DreamShaper",
        )
        self.parser.add_argument("--negative_prompt", help="The negative text prompt")
        self.parser.add_argument("-i", "--input", help="The input image")
        self.parser.add_argument(
            "-o", "--output", help="The output image folder", required=True
        )
        self.parser.add_argument(
            "--name", help="The name of the generated image", default="image"
        )
        self.parser.add_argument("--width", default=768, type=int)
        self.parser.add_argument("--height", default=432, type=int)
        self.parser.add_argument("--batch_size", default=1, type=int)
        self.parser.add_argument("--batch_count", default=1, type=int)
        self.parser.add_argument("--inference_steps", default=50, type=int)
        self.parser.add_argument("--seed", type=int)
        self.parser.add_argument(
            "--controlnet_model", help="The ControlNet model to use"
        )
        self.parser.add_argument(
            "--controlnet_conditioning_scale",
            help="The ControlNet conditioning scale",
            type=float,
            default=1.0,
        )
        self.parser.add_argument(
            "--control_guidance_start",
            help="The ControlNet control guidance start",
            type=float,
            default=0.0,
        )
        self.parser.add_argument(
            "--control_guidance_end",
            help="The ControlNet control guidance end",
            type=float,
            default=1.0,
        )
        self.parser.add_argument(
            "--lora_model", help="A LoRA model to use", nargs="*", default=[]
        )
        self.parser.add_argument(
            "--lora_filename", help="The LoRA file name", nargs="*", default=[]
        )
        self.parser.add_argument(
            "--lora_scale", help="The LoRA scale", nargs="*", default=[], type=float
        )
        self.parser.add_argument(
            "--variations", help="Create image variations", default=False, type=bool
        )

    def run(self, args):
        if args.seed == None:
            seed = random.randint(0, 1000000)
        else:
            seed = args.seed

        if not sys.stdin.isatty():
            args.prompt = sys.stdin.read().splitlines() + args.prompt
        prompt = ", ".join(args.prompt)

        loras = []
        for index, lora_model in enumerate(args.lora_model):
            loras.append(
                {
                    "model": lora_model,
                    "filename": args.lora_filename[index]
                    if index < len(args.lora_filename)
                    else None,
                    "scale": args.lora_scale[index]
                    if index < len(args.lora_scale)
                    else 1.0,
                }
            )

        _logger.info('Creating image for "{0}"'.format(prompt))
        from ..services.image import ImageService

        self.service = ImageService()

        for x in range(args.batch_count):
            path = os.path.join(args.output, args.name)
            pathlib.Path(path).mkdir(parents=True, exist_ok=True)
            s = seed + x * args.batch_size
            if args.input != None:
                if args.controlnet_model != None:
                    self.service.controlnet(
                        args.model,
                        prompt,
                        args.negative_prompt,
                        path,
                        args.width,
                        args.height,
                        args.controlnet_model,
                        args.controlnet_conditioning_scale,
                        args.control_guidance_start,
                        args.control_guidance_end,
                        args.input,
                        loras,
                        s,
                        args.batch_size,
                        args.inference_steps,
                        args.name + "-" + str(x).rjust(3, "0"),
                    )
                elif args.variations:
                    self.service.variations(
                        path,
                        args.width,
                        args.height,
                        args.input,
                        loras,
                        s,
                        args.batch_size,
                        args.inference_steps,
                        args.name + "-" + str(x).rjust(3, "0"),
                    )
                else:
                    self.service.img2img(
                        args.model,
                        prompt,
                        args.negative_prompt,
                        path,
                        args.width,
                        args.height,
                        args.input,
                        loras,
                        s,
                        args.batch_size,
                        args.inference_steps,
                        args.name + "-" + str(x).rjust(3, "0"),
                    )
            else:
                self.service.txt2img(
                    args.model,
                    prompt,
                    args.negative_prompt,
                    path,
                    args.width,
                    args.height,
                    loras,
                    s,
                    args.batch_size,
                    args.inference_steps,
                    args.name + "-" + str(x).rjust(3, "0"),
                )



import os
from unittest.mock import MagicMock

import pytest
import yaml

from giger.cli import CLI
from giger.commands.image import ImageCommand
from giger.commands.prompt import PromptCommand
from giger.commands.roop import RoopCommand
from giger.commands.template import TemplateCommand

__author__ = "Sebastian Krüger"
__copyright__ = "Sebastian Krüger"
__license__ = "MIT"


def test_template(snapshot):
    """CLI Template Tests"""
    snapshots_dir = os.path.join(
        os.path.dirname(__file__), "snapshots", "cli", "template"
    )
    snapshot.snapshot_dir = snapshots_dir

    fixtures = [
        (
            "basic",
            [
                "--config",
                "type=Viking",
                "--data",
                "data/hero.yaml",
                "This is a {{type}}",
            ],
        )
    ]
    for test_name, fixture in fixtures:
        TemplateCommand.run = MagicMock()
        CLI().run(["template"] + fixture)

        snapshot.assert_match(
            yaml.dump(TemplateCommand.run.call_args.args),
            os.path.join(snapshots_dir, test_name + ".yml.snapshot"),
        )


def test_prompt(snapshot):
    """CLI Prompt Tests"""
    snapshots_dir = os.path.join(
        os.path.dirname(__file__), "snapshots", "cli", "prompt"
    )
    snapshot.snapshot_dir = snapshots_dir

    fixtures = [
        ("basic", ["Spawn in a battle", "--time Ancient", "--type", "Comic Book"])
    ]
    for test_name, fixture in fixtures:
        PromptCommand.run = MagicMock()
        CLI().run(["prompt"] + fixture)

        snapshot.assert_match(
            yaml.dump(PromptCommand.run.call_args.args),
            os.path.join(snapshots_dir, test_name + ".yml.snapshot"),
        )


def test_image(snapshot):
    """CLI Image Tests"""
    snapshots_dir = os.path.join(os.path.dirname(__file__), "snapshots", "cli", "image")
    snapshot.snapshot_dir = snapshots_dir

    fixtures = [("basic", ["--output", "out", "--name", "graffiti"])]
    for test_name, fixture in fixtures:
        ImageCommand.run = MagicMock()
        CLI().run(["image"] + fixture)

        snapshot.assert_match(
            yaml.dump(ImageCommand.run.call_args.args),
            os.path.join(snapshots_dir, test_name + ".yml.snapshot"),
        )


def test_roop(snapshot):
    """CLI Roop Tests"""
    snapshots_dir = os.path.join(os.path.dirname(__file__), "snapshots", "cli", "roop")
    snapshot.snapshot_dir = snapshots_dir

    fixtures = [
        (
            "basic",
            ["--source", "face.jpg", "--input", "target.png", "--output", "output.png"],
        )
    ]
    for test_name, fixture in fixtures:
        RoopCommand.run = MagicMock()
        CLI().run(["roop"] + fixture)

        snapshot.assert_match(
            yaml.dump(RoopCommand.run.call_args.args),
            os.path.join(snapshots_dir, test_name + ".yml.snapshot"),
        )



#!/usr/bin/env python

import argparse
import logging
import sys
import warnings

from giger import __version__

from .commands.image import ImageCommand
from .commands.prompt import PromptCommand
from .commands.roop import RoopCommand
from .commands.template import TemplateCommand

warnings.filterwarnings("ignore", category=FutureWarning)


__author__ = "Sebastian Krüger"
__copyright__ = "Sebastian Krüger"
__license__ = "MIT"

_logger = logging.getLogger(__name__)


# ---- CLI ----
# The functions defined in this section are wrappers around the main Python
# API allowing them to be called directly from the terminal as a CLI
# executable/script.


class CLI:
    def parse_args(self, args):
        """
        Parse command line parameters
        """

        parser = argparse.ArgumentParser(
            prog="giger", description="Tools for Stable Diffusion"
        )

        parser.add_argument(
            "--version",
            action="version",
            version=f"giger {__version__}",
        )
        parser.add_argument(
            "-v",
            "--verbose",
            dest="loglevel",
            help="set loglevel to INFO",
            action="store_const",
            const=logging.INFO,
        )
        parser.add_argument(
            "-vv",
            "--very-verbose",
            dest="loglevel",
            help="set loglevel to DEBUG",
            action="store_const",
            const=logging.DEBUG,
        )

        subparsers = parser.add_subparsers(dest="command", required=True)

        self.template = TemplateCommand(subparsers)
        self.prompt = PromptCommand(subparsers)
        self.image = ImageCommand(subparsers)
        self.roop = RoopCommand(subparsers)

        return parser.parse_args(args)

    def setup_logging(self, loglevel):
        """
        Setup basic logging

        Args:
            loglevel (int): minimum loglevel for emitting messages
        """
        logformat = "[%(asctime)s] %(levelname)s - %(name)s - %(message)s"
        logging.basicConfig(
            level=loglevel,
            stream=sys.stdout,
            format=logformat,
            datefmt="%Y-%m-%d %H:%M:%S",
        )

    def run(self, args):
        args = self.parse_args(args)
        self.setup_logging(args.loglevel)
        if args.command == "template":
            self.template.run(args)
        elif args.command == "prompt":
            self.prompt.run(args)
        elif args.command == "image":
            self.image.run(args)
        elif args.command == "roop":
            self.roop.run(args)


def run():
    CLI().run(sys.argv[1:])


if __name__ == "__main__":
    run()



import logging

_logger = logging.getLogger(__name__)


class RoopCommand:
    def __init__(self, parser):
        self.parser = parser.add_parser("roop", help="Swap faces")
        self.parser.add_argument(
            "-s", "--source", help="The source face image", required=True
        )
        self.parser.add_argument("-i", "--input", help="The input image", required=True)
        self.parser.add_argument(
            "-o", "--output", help="The output file", required=True
        )
        self.parser.add_argument("-m", "--model", help="The path to the model")

    def run(self, args):
        _logger.info(
            'Running Roop for "{0}" with "{1}"'.format(args.input, args.source)
        )
        from ..services.roop import RoopService

        RoopService().swap(args.source, args.input, args.output, args.model)



import logging
import sys

from ..services.prompt import PromptService

_logger = logging.getLogger(__name__)


class PromptCommand:
    def __init__(self, parser):
        self.parser = parser.add_parser("prompt", help="Generate prompt")
        self.parser.add_argument("description", nargs="*")
        self.parser.add_argument("--time", choices=PromptService.times())
        self.parser.add_argument("--type", choices=PromptService.types())
        self.parser.add_argument("--background_color")
        self.parser.add_argument(
            "--art_style", choices=PromptService.art_styles(), nargs="*"
        )
        self.parser.add_argument("--artist", choices=PromptService.artists(), nargs="*")
        self.parser.add_argument(
            "--realism", choices=PromptService.realisms(), nargs="*"
        )
        self.parser.add_argument(
            "--rendering_engine", choices=PromptService.rendering_engines(), nargs="*"
        )
        self.parser.add_argument(
            "--lightning_angle", choices=PromptService.lightning_angles(), nargs="*"
        )
        self.parser.add_argument(
            "--lightning_style", choices=PromptService.lightning_styles(), nargs="*"
        )
        self.parser.add_argument(
            "--camera_position", choices=PromptService.camera_positions(), nargs="*"
        )
        self.parser.add_argument("--camera", choices=PromptService.cameras(), nargs="*")
        self.parser.add_argument("--style", choices=PromptService.styles(), nargs="*")
        self.parser.add_argument(
            "--composition", choices=PromptService.compositions(), nargs="*"
        )
        self.parser.add_argument("--iso", choices=PromptService.isos())
        self.parser.add_argument(
            "--resolution", choices=PromptService.resolutions(), nargs="*"
        )
        self.parser.add_argument(
            "--compel_style", choices=PromptService.compel_styles()
        )
        self.parser.add_argument("-o", "--out", help="The txt file to generate")

    def run(self, args):
        if not sys.stdin.isatty():
            args.description = sys.stdin.read().splitlines() + args.description
        args.description = ", ".join(args.description)
        _logger.info('Creating prompt for "{0}"'.format(args.description))
        result = PromptService().generate(args)

        if args.out:
            f = open(args.out, "w")
            f.write(result)
            f.close()
        else:
            print(result)
